package thesis.simulation;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.imageio.ImageIO;

import thesis.gui.Universe;
import thesis.tree.Body;

public final class VideoRecorder {
  
  private ZipOutputStream output;
  private BufferedImage frame;
  private int nextFrameIndex;
  private boolean failed;
  
  public VideoRecorder(String filename) throws FileNotFoundException {
    this.output = new ZipOutputStream(new FileOutputStream(filename));
    this.frame = new BufferedImage(Universe.WIDTH, Universe.HEIGHT,
        BufferedImage.TYPE_INT_RGB);
    
    this.nextFrameIndex = 0;
    this.failed = false;
  }
  
  public void writeFrame(List<Body> bodies) {
    if (!failed) {
      Graphics g = frame.createGraphics();
      g.setColor(Color.BLACK);
      g.fillRect(0, 0, frame.getWidth(), frame.getHeight());
      g.dispose();

      AffineTransform transform = Universe.getTransform();
      for (Body body : bodies) {
        Point2D point = transform.transform(new Point2D.Float(
            body.getPosition().getX(), body.getPosition().getY()), null);

        int x = (int)point.getX();
        int y = (int)point.getY();
        if (x >= 0 && x < frame.getWidth() && y >= 0 && y < frame.getHeight()) {
          frame.setRGB(x, y, Color.WHITE.getRGB());
        }
      }

      StringBuilder filename = new StringBuilder(
          Integer.toString(nextFrameIndex++));

      while (filename.length() < 8) {
        filename.insert(0, '0');
      }
      try {
        output.putNextEntry(new ZipEntry(filename.append(".png").toString()));
        ImageIO.write(frame, "png", output);
        output.closeEntry();
      } catch (IOException e) {
        failed = true;
      }
    }
  }
  
  public BufferedImage getFrame() {
    return frame;
  }
  
  public void close() {
    try {
      output.close();
    } catch (IOException e) {
      failed = true;
    }
  }
  
  public boolean hasFailed() {
    return failed;
  }

}
